##Library Header.
# $Id: $
# Copyright  2003-2009 by IXIA
# All Rights Reserved.
#
# Name:
#    ixia_rsvp_api.tcl
#
# Purpose:
#     A script development library containing RSVP APIs for test automation
#     with the Ixia chassis.
#
# Author:
#    Hasmik Shakaryan
#
# Usage:
#    package require Ixia
#
# Description:
#    The procedures contained within this library include:
#
#    - emulation_rsvp_config
#    - emulation_rsvp_tunnel_config
#    - emulation_rsvp_control
#    - emulation_rsvp_info
#    - emulation_rsvp_tunnel_info
#
# Requirements:
#     ixiaapiutils.tcl , a library containing TCL utilities
#     parseddashedargs.tcl , a library containing the proceDescr and
#     parsedashedargds.tcl
#
# Variables:
#
# Keywords:
#
# Category:
#
################################################################################
#                                                                              #
#                                LEGAL  NOTICE:                                #
#                                ==============                                #
# The following code and documentation (hereinafter "the script") is an        #
# example script for demonstration purposes only.                              #
# The script is not a standard commercial product offered by Ixia and have     #
# been developed and is being provided for use only as indicated herein. The   #
# script [and all modifications, enhancements and updates thereto (whether     #
# made by Ixia and/or by the user and/or by a third party)] shall at all times #
# remain the property of Ixia.                                                 #
#                                                                              #
# Ixia does not warrant (i) that the functions contained in the script will    #
# meet the user's requirements or (ii) that the script will be without         #
# omissions or error-free.                                                     #
# THE SCRIPT IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, AND IXIA        #
# DISCLAIMS ALL WARRANTIES, EXPRESS, IMPLIED, STATUTORY OR OTHERWISE,          #
# INCLUDING BUT NOT LIMITED TO ANY WARRANTY OF MERCHANTABILITY AND FITNESS FOR #
# A PARTICULAR PURPOSE OR OF NON-INFRINGEMENT.                                 #
# THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SCRIPT  IS WITH THE #
# USER.                                                                        #
# IN NO EVENT SHALL IXIA BE LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING   #
# OUT OF THE USE OF, OR THE INABILITY TO USE THE SCRIPT OR ANY PART THEREOF,   #
# INCLUDING BUT NOT LIMITED TO ANY LOST PROFITS, LOST BUSINESS, LOST OR        #
# DAMAGED DATA OR SOFTWARE OR ANY INDIRECT, INCIDENTAL, PUNITIVE OR            #
# CONSEQUENTIAL DAMAGES, EVEN IF IXIA HAS BEEN ADVISED OF THE POSSIBILITY OF   #
# SUCH DAMAGES IN ADVANCE.                                                     #
# Ixia will not be required to provide any software maintenance or support     #
# services of any kind (e.g., any error corrections) in connection with the    #
# script or any part thereof. The user acknowledges that although Ixia may     #
# from time to time and in its sole discretion provide maintenance or support  #
# services for the script, any such services are subject to the warranty and   #
# damages limitations set forth herein and will not obligate Ixia to provide   #
# any additional maintenance or support services.                              #
#                                                                              #
################################################################################

proc ::ixia::emulation_rsvp_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rsvp_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable rsvp_handles_array

    ::ixia::utrackerLog $procName $args

    keylset returnList status $::SUCCESS

    # Arguments
    set man_args {
        -mode        CHOICES create delete disable enable modify
    }
    set opt_args {
        -port_handle                    REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -handle
        -interface_handle
        -actual_restart_time            RANGE       50-4294967295
                                        DEFAULT     15000
        -bfd_registration               CHOICES 0 1
                                        DEFAULT 0
        -bundle_msg_sending             CHOICES 0 1
                                        DEFAULT 0
        -count                          NUMERIC
                                        DEFAULT 1
        -enable_bgp_over_lsp            CHOICES     0 1
                                        DEFAULT     1
        -gateway_ip_addr                IP
        -gateway_ip_addr_step           IP
                                        DEFAULT 0.0.1.0
        -graceful_restart               CHOICES     0 1
                                        DEFAULT     0
        -graceful_restart_helper_mode   CHOICES 0 1
                                        DEFAULT 0
        -graceful_restart_recovery_time RANGE       0-4294967295
                                        DEFAULT     30000
        -graceful_restart_restart_time  RANGE       100-4294967295
                                        DEFAULT     5000
        -graceful_restart_start_time    RANGE       50-4294967295
                                        DEFAULT     30000
        -graceful_restart_up_time       RANGE       50-4294967295
                                        DEFAULT     30000
        -graceful_restarts_count        RANGE       0-65535
                                        DEFAULT     0
        -hello_tlvs                     ANY
        -intf_ip_addr               IP
                                    DEFAULT 0.0.0.0
        -intf_prefix_length         RANGE   1-32
                                    DEFAULT 24
        -intf_ip_addr_step          IP
                                    DEFAULT 0.0.1.0
        -neighbor_intf_ip_addr      IP
                                    DEFAULT 0.0.0.0
        -neighbor_intf_ip_addr_step IP
                                    DEFAULT 0.0.1.0
        -refresh_interval           NUMERIC
        -refresh_retry_count        NUMERIC
        -summary_refresh            CHOICES 0 1
                                    DEFAULT 0
        -srefresh_interval          NUMERIC
        -hello_msgs                 CHOICES 0 1
        -hello_interval             RANGE   1-65535
        -hello_retry_count          RANGE   1-255
        -refresh_reduction          CHOICES 0 1
                                    DEFAULT 0
        -reliable_delivery          CHOICES 0 1
                                    DEFAULT 0
        -bundle_msgs                CHOICES 0 1
                                    DEFAULT 0
        -resv_confirm               CHOICES 0 1
        -record_route               CHOICES 0 1
        -ttl                        CHOICES 1 64
        -egress_label_mode          CHOICES nextlabel imnull exnull
        -max_label_value            RANGE   1-1048575
        -min_label_value            RANGE   1-1048575
        -vpi                        RANGE   0-255
        -vci                        RANGE   0-65535
        -vpi_step                   RANGE   0-255
        -vci_step                   RANGE   0-65535
        -atm_encapsulation          CHOICES VccMuxIPV4Routed
                                    CHOICES VccMuxIPV6Routed
                                    CHOICES VccMuxBridgedEthernetFCS
                                    CHOICES VccMuxBridgedEthernetNoFCS
                                    CHOICES LLCRoutedCLIP
                                    CHOICES LLCBridgedEthernetFCS
                                    CHOICES LLCBridgedEthernetNoFCS
        -vlan_id                    RANGE   0-4095
        -vlan_id_mode               CHOICES fixed increment
                                    DEFAULT increment
        -vlan_id_step               RANGE   0-4095
                                    DEFAULT 1
        -vlan_user_priority         RANGE   0-7
                                    DEFAULT 0
        -path_state_refresh_timeout RANGE   1-65535
        -resv_state_refresh_timeout RANGE   1-65535
        -router_alert               CHOICES 1
        -path_state_timeout_count   RANGE   1-255
        -resv_state_timeout_count   RANGE   1-255
        -reset
        -ip_version                 CHOICES 4
                                    DEFAULT 4
        -mac_address_init           MAC
        -mac_address_step           MAC
                                    DEFAULT 0000.0000.0001
        -no_write                   FLAG
        -vlan                       CHOICES 0 1
        -writeFlag                  CHOICES write nowrite
        -override_existence_check   CHOICES 0 1
                                    DEFAULT 0
        -override_tracking          CHOICES 0 1
                                    DEFAULT 0
    }

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rsvp_config $args $man_args $opt_args]
        if {[keylget returnList status] == $::FAILURE} {
            keylset returnList log "ERROR in $procName:\
                    [keylget returnList log]"
        }
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_rsvp_tunnel_config { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rsvp_tunnel_config $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable rsvp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    keylset returnList status $::SUCCESS

    # Arguments
    set man_args {
        -mode                               CHOICES create delete modify
    }
    set opt_args {
        -adspec                             CHOICES 1
        -avoid_node_id                      VCMD ::ixia::isValidIPAddress
        -count                              NUMERIC
        -egress_behavior                        CHOICES always_use_configured_style
                                                CHOICES use_se_when_indicated_in_session_attribute
                                                DEFAULT always_use_configured_style
        -egress_ip_addr                     IP
                                            DEFAULT 0.0.0.0
        -egress_ip_count                    NUMERIC
        -egress_ip_step                     IP
                                            DEFAULT 0.0.1.0
        -egress_leaf_ip_count               NUMERIC
                                            DEFAULT 1
        -egress_leaf_range_count            NUMERIC
                                            DEFAULT 1
        -egress_leaf_range_param_type       CHOICES list single_value
                                            DEFAULT single_value
        -egress_leaf_range_step             IP
                                            DEFAULT 1.0.0.0
        -emulation_type                         CHOICES rsvpte rsvptep2mp
                                                DEFAULT rsvpte
        -enable_append_connected_ip             CHOICES 0 1
                                                DEFAULT 1
        -enable_prepend_tunnel_leaf_ip          CHOICES 0 1
                                                DEFAULT 1
        -enable_prepend_tunnel_head_ip          CHOICES 0 1
                                                DEFAULT 1
        -enable_send_as_rro                     CHOICES 0 1
                                                DEFAULT 1
        -enable_send_as_srro                    CHOICES 0 1
                                                DEFAULT 0
        -ero                                CHOICES 0 1
                                            DEFAULT 0
        -ero_dut_pfxlen                     RANGE 1-32
        -ero_list_as_num                    NUMERIC
        -ero_list_ipv4                      IP
        -ero_list_loose                     CHOICES 0 1
        -ero_list_pfxlen                    RANGE 1-128
        -ero_list_type                      CHOICES ipv4 as
                                            DEFAULT ipv4
        -ero_mode                           CHOICES loose strict none
                                            DEFAULT loose
        -explicit_traffic_item              CHOICES 0 1
                                            DEFAULT 0
        -extended_tunnel_id_type            CHOICES egress ingress custom
        -facility_backup                    CHOICES 0 1
                                            DEFAULT 0
        -fast_reroute                       CHOICES 0 1
                                            DEFAULT 0
        -fast_reroute_bandwidth
        -fast_reroute_exclude_any
        -fast_reroute_holding_priority      RANGE 0-255
                                            DEFAULT 7
        -fast_reroute_hop_limit             RANGE 0-255
                                            DEFAULT 3
        -fast_reroute_include_all
        -fast_reroute_include_any
        -fast_reroute_setup_priority        RANGE 0-255
                                            DEFAULT 7
        -h2l_info_dut_hop_type                  CHOICES strict loose
                                                DEFAULT loose
        -h2l_info_dut_prefix_length             RANGE   1-32
                                                DEFAULT 32
        -h2l_info_enable_append_tunnel_leaf     CHOICES 0 1
                                                DEFAULT 1
        -h2l_info_enable_prepend_dut            CHOICES 0 1
                                                DEFAULT 1
        -h2l_info_enable_send_as_ero            CHOICES 0 1
                                                DEFAULT 1
        -h2l_info_enable_send_as_sero           CHOICES 0 1
                                                DEFAULT 0
        -h2l_info_ero_sero_list                 ANY
        -h2l_info_tunnel_leaf_count             RANGE   1-4294967295
                                                DEFAULT 1
        -h2l_info_tunnel_leaf_hop_type          CHOICES strict loose
                                                DEFAULT loose
        -h2l_info_tunnel_leaf_ip_start          IP
                                                DEFAULT 0.0.0.0
        -h2l_info_tunnel_leaf_prefix_length     RANGE   1-32
                                                DEFAULT 32
        -handle
        -head_traffic_ip_type                   CHOICES ipv4 ipv6
                                                DEFAULT ipv4
        -head_traffic_ip_count                  RANGE   1-4294967295
                                                DEFAULT 1
        -head_traffic_start_ip                  IP
        -head_traffic_inter_tunnel_ip_step    IP
        -ingress_bandwidth                  DECIMAL
        -ingress_ip_addr                    IP
                                            DEFAULT 0.0.0.0
        -ingress_ip_count                   NUMERIC
        -ingress_ip_step                    IP
                                            DEFAULT 0.0.1.0
        -ingress_enable_interface_creation  CHOICES 0 1
                                            DEFAULT 1
        -lsp_id_count                       RANGE 0-65535
                                            DEFAULT 1
        -lsp_id_start                       RANGE 0-65535
                                            DEFAULT 0
        -lsp_id_step                        RANGE 0-65535
                                            DEFAULT 1
        -no_write                           FLAG
        -one_to_one_backup                  CHOICES 0 1
                                            DEFAULT 0
        -p2mp_id                                ANY
                                                DEFAULT "0.0.0.1"
        -p2mp_id_step                           ANY
                                                DEFAULT "0.0.0.0"
        -path_error_tlv                         ANY
        -path_tear_tlv                          ANY
        -path_tlv                               ANY
        -plr_id                             VCMD ::ixia::isValidIPAddress
        -port_handle                        REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -record_route                           CHOICES 0 1
        -reservation_error_tlv                  ANY
        -reservation_style                      CHOICES ff se
                                                DEFAULT se
        -reservation_tear_tlv                   ANY
        -reservation_tlv                        ANY
        -rro                                CHOICES 0 1
                                            DEFAULT 0
        -rro_list_ctype                     RANGE 0-255
        -rro_list_flags                     RANGE 0-255
        -rro_list_ipv4                      IP
        -rro_list_label                     NUMERIC
        -rro_list_type                      CHOICES ipv4 label
                                            DEFAULT ipv4
        -rsvp_behavior                      CHOICES rsvpIngress rsvpEgress
                                            DEFAULT rsvpEgress
        -send_detour
        -sender_tspec_max_pkt_size          NUMERIC
        -sender_tspec_min_policed_size      NUMERIC
        -sender_tspec_peak_data_rate        DECIMAL
        -sender_tspec_token_bkt_rate        DECIMAL
        -sender_tspec_token_bkt_size        DECIMAL
        -session_attr                       CHOICES 1
        -session_attr_setup_priority        RANGE 0-255
        -session_attr_hold_priority         RANGE 0-255
        -session_attr_name
        -session_attr_local_protect         CHOICES 0 1
        -session_attr_label_record          CHOICES 0 1
        -session_attr_se_style              CHOICES 0 1
        -session_attr_bw_protect            CHOICES 0 1
        -session_attr_node_protect          CHOICES 0 1
        -session_attr_flags                 RANGE 0-255
        -session_attr_resource_affinities   CHOICES 0 1
                                            DEFAULT 0
        -session_attr_ra_exclude_any
        -session_attr_ra_include_any
        -session_attr_ra_include_all
        -session_attr_reroute               CHOICES 0 1
        -tail_traffic_ip_type                   CHOICES ipv4 ipv6
                                                DEFAULT ipv4
        -tail_traffic_ip_count                  RANGE   1-4294967295
                                                DEFAULT 1
        -tail_traffic_start_ip                  IP
        -tail_traffic_inter_tunnel_ip_step      IP
        -tunnel_id_start                    RANGE 0-65535
                                            DEFAULT 0
        -tunnel_id_count                    RANGE 0-65535
                                            DEFAULT 1
        -tunnel_id_step                     RANGE 0-65535
                                            DEFAULT 1
        -tunnel_pool_handle
        -writeFlag                          CHOICES write nowrite
                                            DEFAULT write
    }

    # Items that we do not support or they are ON by default
    # adspec                    - enabled by default
    # adspec_comp_mtu           - default to 0
    # adspec_csum               - default to 0
    # adspec_ctot               - default to 0
    # adspec_ctrl_load          - default to 0
    # adspec_dsum               - default to 0
    # adspec_dtot               - default to 0
    # adspec_est_path_bw        - default to 0
    # adspec_guaranteed_svc     - default to 0
    # adspec_hop_count          - default to 0
    # adspec_min_path_latency   - default to 0
    # destination_ip            - Ixia doesn't support
    # duration                  - Will not be used
    # extended_tunnel_id        - Ixia doesn't support
    # extended_tunnel_id_type   - Ixia doesn't support
    # interval                  - Will not be used
    # sender_adspec             - N/A
    # sender_cos                - Not user configurable
    # session_attr              - On by default
    # ttl                       - N/A It is not user configurable.  By default
    #                                 it is 1 for hello messages and for
    #                                 anything else it is 64.

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rsvp_tunnel_config $args $man_args $opt_args]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_rsvp_control { args } {
    variable executeOnTclServer

    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rsvp_control $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable rsvp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set man_args {
        -mode CHOICES stop start restart sub_lsp_up sub_lsp_down
    }

    set opt_args {
        -handle
        -port_handle REGEXP  ^[0-9]+/[0-9]+/[0-9]+$
        -teardown
        -restore
    }

	if {[isUNIX] && [info exists ::ixTclSvrHandle]} {
        set retValueClicks [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock clicks}"]
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValueSeconds [eval "::ixia::SendToIxTclServer $::ixTclSvrHandle {clock seconds}"]
    } else {
        set retValueClicks [clock clicks]
        set retValueSeconds [clock seconds]
    }
    keylset returnList clicks [format "%u" $retValueClicks]
    keylset returnList seconds [format "%u" $retValueSeconds]

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rsvp_control $args $man_args $opt_args]
        keylset returnList clicks [format "%u" $retValueClicks]
        keylset returnList seconds [format "%u" $retValueSeconds]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
    return $returnList
}


proc ::ixia::emulation_rsvp_info { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rsvp_info $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable rsvp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set opt_args {
        -mode        CHOICES stats clear_stats settings neighbors labels
                     DEFAULT stats
        -port_handle REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
    }

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rsvp_info $args $opt_args]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
	
    return $returnList
}


proc ::ixia::emulation_rsvp_tunnel_info { args } {
    variable executeOnTclServer
    
    set procName [lindex [info level [info level]] 0]
	
    ::ixia::logHltapiCommand $procName $args
    
    if {$::ixia::executeOnTclServer} {
        if {![info exists ::ixTclSvrHandle]} {
            keylset returnList status $::FAILURE
            keylset returnList log "Not connected to TclServer."
            return $returnList
        }
        set retValue [eval ::ixia::SendToIxTclServer $::ixTclSvrHandle \
                \{::ixia::emulation_rsvp_tunnel_info $args\}]
        
        set startIndex [string last "\r" $retValue]
        if {$startIndex >= 0} {
            set retData [string range $retValue [expr $startIndex + 1] end]
            return $retData
        } else {
            return $retValue
        }
    }
    
    variable rsvp_handles_array
    
    ::ixia::utrackerLog $procName $args
    
    # Arguments
    set opt_args {
        -port_handle REGEXP ^[0-9]+/[0-9]+/[0-9]+$
        -handle
        -info_type   CHOICES assigned_info received_info both
                     DEFAULT both
    }

    variable new_ixnetwork_api
    if {[info exists new_ixnetwork_api] && $new_ixnetwork_api} {
        set returnList [::ixia::ixnetwork_rsvp_tunnel_info $args $opt_args]
        return $returnList
    } else {
		set returnList [::ixia::use_ixtclprotocol]
		keylset returnList log "ERROR in $procName: [keylget returnList log]"
	}
    
    return $returnList
}
